package com.benyun.core.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.benyun.core.dao.TopologicalBuildWdDao;
import com.benyun.core.dao.TopologicalBuildWdStatisticsCategoryDao;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.entity.*;
import com.benyun.core.entity.bo.BuildWdCategoryCount;
import com.benyun.core.entity.bo.StoreWdCategoryCount;
import com.benyun.core.entity.bo.StoreWdCategoryCountBody;
import com.benyun.core.entity.vo.TopologicalWdAceeptVo;
import com.benyun.core.service.TopologicalBuildWdSerevice;
import com.benyun.core.utils.DistanceUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
@Transactional
public class TopologicalBuildWdSereviceImpl implements TopologicalBuildWdSerevice {

    @Autowired
    TopologicalBuildWdDao topologicalBuildWdDao;

    @Autowired
    TopologicalBuildWdStatisticsCategoryDao topologicalBuildWdStatisticsCategoryDao;

    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    @Qualifier("addrCodeMap")
    HashMap<String,String> addrCodeMap;

    @Override
    public List<BuildWdCategoryCount> buildingLevel(TopologicalWdAceeptVo topologicalWdAceeptVo) {
        QueryWrapper<TopologicalBuildWdStatisticsCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("center_wd_id",topologicalWdAceeptVo.getCenterWdId());
        queryWrapper.and(wrapper -> {
            wrapper.eq("radius",topologicalWdAceeptVo.getRadius());
        });

        List<BuildWdCategoryCount> list = new ArrayList<>();
        TopologicalBuildWdStatisticsCategory topologicalBuildWdStatisticsCategory = topologicalBuildWdStatisticsCategoryDao.selectOne(queryWrapper);
        String propertyTypeStatistics = topologicalBuildWdStatisticsCategory.getPropertyTypeStatistics();
        if (propertyTypeStatistics == null)
            return list;

        int total = 0;
        for (String s : propertyTypeStatistics.split(";")) {
            BuildWdCategoryCount buildWdCategoryCount = new BuildWdCategoryCount();
            String[] split = s.split(":");
            if (split.length !=2)
                continue;
            buildWdCategoryCount.setName(split[0]);
            buildWdCategoryCount.setCount(Integer.parseInt(split[1]));
            total+=buildWdCategoryCount.getCount();
            list.add(buildWdCategoryCount);
        }
        for (BuildWdCategoryCount buildWdCategoryCount : list) {
            BigDecimal bigDecimal = new BigDecimal((double) buildWdCategoryCount.getCount() / total);
            double v = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            buildWdCategoryCount.setRadio(v);
        }
        return list;
    }

    @Override
    public List<TopologicalBuildingWd> list(TopologicalWdAceeptVo topologicalWdAceeptVo) {

        //return pageInfo;
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();

        //条件构造
        queryWrapper.eq("wd_type_code","3");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.likeRight("geo_hash",topologicalWdAceeptVo.getGeoHash().substring(0,5));
        });

        //查询结果
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);

        List<TopologicalBuildingWd> list = new ArrayList<>();  //返回结果集
        GlobalCoordinates source = new GlobalCoordinates(topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getLng());
        for (WdInfo wdInfo : wdInfos) {
            if(list.size() >= topologicalWdAceeptVo.getSize())
                break;

            //1.计算距离
            GlobalCoordinates target = new GlobalCoordinates(wdInfo.getLat().doubleValue(), wdInfo.getLng().doubleValue());
            double meter2 = DistanceUtil.getDistanceMeter(source, target, Ellipsoid.WGS84);

            if(meter2 <= topologicalWdAceeptVo.getRadius()){
                TopologicalBuildingWd topologicalBuildingWd = new TopologicalBuildingWd(wdInfo);
                topologicalBuildingWd.setAddrCodeInfo(addrCodeMap.get(topologicalBuildingWd.getAddrCode()));
                BigDecimal bigDecimal = new BigDecimal(meter2);
                topologicalBuildingWd.setDistance(bigDecimal.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue());

                //补充公司数据
                //....

                list.add(topologicalBuildingWd);
            }
        }
        return list;
    }
}
