package com.benyun.core.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.benyun.core.dao.TopologicalEnterpriseWdDao;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.entity.TopologicalEnterpriseWd;
import com.benyun.core.entity.TopologicalStoreWd;
import com.benyun.core.entity.WdInfo;
import com.benyun.core.entity.vo.TopologicalWdAceeptVo;
import com.benyun.core.service.TopologicalEnterpriseWdService;
import com.benyun.core.utils.DistanceUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class TopologicalEnterpriseWdServiceImpl implements TopologicalEnterpriseWdService {
    @Autowired
    TopologicalEnterpriseWdDao topologicalEnterpriseWdDao;
    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    @Qualifier("addrCodeMap")
    HashMap<String,String> addrCodeMap;
    @Override
    public List<TopologicalEnterpriseWd> list(TopologicalWdAceeptVo topologicalWdAceeptVo) {
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();

        //条件构造
        queryWrapper.eq("wd_type_code","5");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.likeRight("geo_hash",topologicalWdAceeptVo.getGeoHash().substring(0,5));
        });

        //查询结果
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);

        List<TopologicalEnterpriseWd> list = new ArrayList<>();  //返回结果集
        GlobalCoordinates source = new GlobalCoordinates(topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getLng());
        for (WdInfo wdInfo : wdInfos) {
            if(list.size() >= topologicalWdAceeptVo.getSize())
                break;

            //1.计算距离
            GlobalCoordinates target = new GlobalCoordinates(wdInfo.getLat().doubleValue(), wdInfo.getLng().doubleValue());
            double meter2 = DistanceUtil.getDistanceMeter(source, target, Ellipsoid.WGS84);

            if(meter2 <= topologicalWdAceeptVo.getRadius()){
                TopologicalEnterpriseWd topologicalEnterpriseWd = new TopologicalEnterpriseWd(wdInfo);
                topologicalEnterpriseWd.setAddrCodeInfo(addrCodeMap.get(topologicalEnterpriseWd.getAddrCode()));
                BigDecimal bigDecimal = new BigDecimal(meter2);
                topologicalEnterpriseWd.setDistance(bigDecimal.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue());

                //补充公司数据
                //....

                list.add(topologicalEnterpriseWd);
            }
        }
        return list;
    }
}
