package com.benyun.core.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.benyun.core.dao.TopologicalHouseWdDao;
import com.benyun.core.dao.TopologicalHouseWdStatisticsDataDao;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.entity.TopologicalHouseWd;
import com.benyun.core.entity.TopologicalHouseWdStatisticsData;
import com.benyun.core.entity.TopologicalStoreWd;
import com.benyun.core.entity.WdInfo;
import com.benyun.core.entity.vo.TopologicalWdAceeptVo;
import com.benyun.core.service.TopologicalHouseWdService;
import com.benyun.core.utils.DistanceUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class TopologicalHouseServiceWdImpl implements TopologicalHouseWdService {
    @Autowired
    TopologicalHouseWdDao topologicalHouseWdDao;

    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    TopologicalHouseWdStatisticsDataDao topologicalHouseWdStatisticsDataDao;

    @Autowired
    @Qualifier("addrCodeMap")
    HashMap<String,String> addrCodeMap;

    @Override
    public List<TopologicalHouseWd> list(TopologicalWdAceeptVo topologicalWdAceeptVo) {
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();

        //条件构造
        queryWrapper.eq("wd_type_code","2");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.likeRight("geo_hash",topologicalWdAceeptVo.getGeoHash().substring(0,5));
        });

        //查询结果
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);

        List<TopologicalHouseWd> list = new ArrayList<>();  //返回结果集
        GlobalCoordinates source = new GlobalCoordinates(topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getLng());
        for (WdInfo wdInfo : wdInfos) {
            if(list.size() >= topologicalWdAceeptVo.getSize())
                break;

            //1.计算距离
            GlobalCoordinates target = new GlobalCoordinates(wdInfo.getLat().doubleValue(), wdInfo.getLng().doubleValue());
            double meter2 = DistanceUtil.getDistanceMeter(source, target, Ellipsoid.WGS84);

            if(meter2 <= topologicalWdAceeptVo.getRadius()){
                TopologicalHouseWd topologicalHouseWd = new TopologicalHouseWd(wdInfo);
                topologicalHouseWd.setAddrCodeInfo(addrCodeMap.get(topologicalHouseWd.getAddrCode()));
                BigDecimal bigDecimal = new BigDecimal(meter2);
                topologicalHouseWd.setDistance(bigDecimal.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue());

                //补充门店数据
                //....

                list.add(topologicalHouseWd);
            }
        }
        return list;
    }

    @Override
    public TopologicalHouseWdStatisticsData info(TopologicalWdAceeptVo topologicalWdAceeptVo) {
        QueryWrapper<TopologicalHouseWdStatisticsData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("center_wd_id",topologicalWdAceeptVo.getCenterWdId()).and(topologicalHouseWdStatisticsDataQueryWrapper -> {
            topologicalHouseWdStatisticsDataQueryWrapper.eq("radius",Integer.valueOf(topologicalWdAceeptVo.getRadius()));
        });
        TopologicalHouseWdStatisticsData topologicalHouseWdStatisticsData = topologicalHouseWdStatisticsDataDao.selectOne(queryWrapper);
        return topologicalHouseWdStatisticsData;
    }
}
