package com.benyun.core.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.benyun.core.dao.TopologicalStoreWdDao;
import com.benyun.core.dao.TopologicalStoreWdStatisticsCategoryDao;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.entity.ManageType;
import com.benyun.core.entity.TopologicalStoreWd;
import com.benyun.core.entity.TopologicalStoreWdStatisticsCategory;
import com.benyun.core.entity.WdInfo;
import com.benyun.core.entity.bo.StoreWdCategoryCount;
import com.benyun.core.entity.bo.StoreWdCategoryCountBody;
import com.benyun.core.entity.vo.TopologicalWdAceeptVo;
import com.benyun.core.service.TopologicalStoreWdService;
import com.benyun.core.utils.DistanceUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class TopologicalStoreWdServiceImpl implements TopologicalStoreWdService {

    @Autowired
    TopologicalStoreWdStatisticsCategoryDao storeWdStatisticsCategoryDao;

    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    @Qualifier("ManageType")
    HashMap<String, ManageType> manageType;

    @Autowired
    @Qualifier("addrCodeMap")
    HashMap<String,String> addrCodeMap;



    @Override
    public List<TopologicalStoreWd> list(TopologicalWdAceeptVo topologicalWdAceeptVo) {
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();

        //条件构造
        queryWrapper.eq("wd_type_code","1");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.likeRight("geo_hash",topologicalWdAceeptVo.getGeoHash().substring(0,5));
        });

        //查询结果
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);

        List<TopologicalStoreWd> list = new ArrayList<>();  //返回结果集
        GlobalCoordinates source = new GlobalCoordinates(topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getLng());
        for (WdInfo wdInfo : wdInfos) {
            if(list.size() >= topologicalWdAceeptVo.getSize())
                break;

            //1.计算距离
            GlobalCoordinates target = new GlobalCoordinates(wdInfo.getLat().doubleValue(), wdInfo.getLng().doubleValue());
            double meter2 = DistanceUtil.getDistanceMeter(source, target, Ellipsoid.WGS84);

            if(meter2 <= topologicalWdAceeptVo.getRadius()){
                TopologicalStoreWd topologicalStoreWd = new TopologicalStoreWd(wdInfo);
                topologicalStoreWd.setAddrCodeInfo(addrCodeMap.get(topologicalStoreWd.getAddrCode()));
                BigDecimal bigDecimal = new BigDecimal(meter2);
                topologicalStoreWd.setDistance(bigDecimal.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue());

                //补充门店数据
                //....

                list.add(topologicalStoreWd);
            }
        }
        return list;
    }

    //TODO 未返回total
    @Override
    public List<StoreWdCategoryCount> category(TopologicalWdAceeptVo topologicalWdAceeptVo) {

        List<StoreWdCategoryCount> storeWdCategoryCounts = new ArrayList<>();

        int total = 0;
        List<TopologicalStoreWdStatisticsCategory> statisticsCategories = storeWdStatisticsCategoryDao.category(topologicalWdAceeptVo);
        for (TopologicalStoreWdStatisticsCategory statisticsCategory : statisticsCategories) {
            StoreWdCategoryCount storeWdCategoryCount = new StoreWdCategoryCount();

            String midManageTypeCode = statisticsCategory.getMidManageTypeCode();
            String[] split1 = midManageTypeCode.split(":");
            storeWdCategoryCount.setManageTypeCode(split1[0]+"00");
            ManageType manageType1 = manageType.get(storeWdCategoryCount.getManageTypeCode());
            storeWdCategoryCount.setName(manageType1.getMidCategory());
            storeWdCategoryCount.setCount(Integer.parseInt(split1[1]));
            total+=storeWdCategoryCount.getCount();

            //拆分
            String subCategoryCodeStatistics = statisticsCategory.getSubCategoryCodeStatistics();
            List<StoreWdCategoryCountBody> storeWdCategoryCountBodyList = new ArrayList<>();
            for (String s : subCategoryCodeStatistics.split(";")) {
                String[] split = s.split(":");
                if (split.length !=2)
                    continue;
                if(split[0].equals(storeWdCategoryCount.getManageTypeCode()))
                    break;

                StoreWdCategoryCountBody storeWdCategoryCountBody = new StoreWdCategoryCountBody();
                storeWdCategoryCountBody.setManageTypeCode(split[0]);
                ManageType manageType2 = manageType.get(storeWdCategoryCountBody.getManageTypeCode());
                storeWdCategoryCountBody.setName(manageType2.getSubCategory());
                storeWdCategoryCountBody.setCount(Integer.parseInt(split[1]));
                BigDecimal bigDecimal = new BigDecimal((double) storeWdCategoryCount.getCount() / storeWdCategoryCountBody.getCount());
                double v = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                storeWdCategoryCountBody.setRadio(v);
                storeWdCategoryCountBodyList.add(storeWdCategoryCountBody);
            }
            storeWdCategoryCount.setStoreWdCategoryCountBodyList(storeWdCategoryCountBodyList);
            storeWdCategoryCounts.add(storeWdCategoryCount);
        }

        return storeWdCategoryCounts;
    }
}
